﻿using NUnit.Framework;
using Sunset.Library;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SunsetLibrary.Tests
{
    [TestFixture]
    public class ContentParserTests
    {
        string goodResult = "{\"results\":{\"sunrise\":\"6:39:53 AM\",\"sunset\":\"6:48:32 PM\",\"solar_noon\":\"12:44:12 PM\",\"day_length\":\"12:08:38.8300000\"},\"status\":\"OK\"}";
        string badResult = "{\"results\":null,\"status\":\"ERROR\"}";

        [Test]
        public void CheckStatus_OnValidStatus_ReturnsTrue()
        {
            bool result = ContentParser.CheckStatus(goodResult);
            Assert.IsTrue(result);
        }

        [Test]
        public void CheckStatus_OnErrorStatus_ReturnsFalse()
        {
            bool result = ContentParser.CheckStatus(badResult);
            Assert.IsFalse(result);
        }

        [Test]
        public void ParseSunsetString_OnValidData_ReturnsExpectedValue()
        {
            string expected = "6:48:32 PM";
            string result = ContentParser.ParseSunsetString(goodResult);
            Assert.AreEqual(expected, result);
        }

        [Test]
        public void ParseSunriseString_OnValidData_ReturnsExpectedValue()
        {
            string expected = "6:39:53 AM";
            string result = ContentParser.ParseSunriseString(goodResult);
            Assert.AreEqual(expected, result);
        }

        [Test]
        public void ParseSunsetString_OnErrorDate_ThrowsArgumentException()
        {
            try
            {
                ContentParser.ParseSunsetString(badResult);
                Assert.Fail("ArgumentException not thrown");
            }
            catch (ArgumentException)
            {
                Assert.Pass();
            }
        }

        [Test]
        public void ParseSunriseString_OnErrorDate_ThrowsArgumentException()
        {
            try
            {
                ContentParser.ParseSunriseString(badResult);
                Assert.Fail("ArgumentException not thrown");
            }
            catch (ArgumentException)
            {
                Assert.Pass();
            }
        }
    }
}
