﻿using Newtonsoft.Json;
using Sunset.Interface;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading.Tasks;

namespace Sunset.Library
{
    public class SunsetCalculator : ISunsetCalculator
    {
        string cacheData = string.Empty;
        DateTime cacheDataDate = DateTime.MinValue;

        private ISolarService service;
        public ISolarService Service
        {
            get
            {
                if (service == null)
                    return new SolarService();
                return service;
            }
            set { service = value; }
        }


        public DateTime GetSunrise(DateTime date)
        {
            throw new NotImplementedException();
        }

        public DateTime GetSunset(DateTime date)
        {
            if(cacheDataDate != date)
            {
                cacheData = Service.GetServiceData(date);
                cacheDataDate = date;
            }

            string sunsetTime = ParseSunsetString(cacheData);
            DateTime sunset = ToLocalTime(date, sunsetTime);
            return sunset;
        }

        public static bool CheckStatus(string jsonContent)
        {
            dynamic content = JsonConvert.DeserializeObject(jsonContent);
            return content.status == "OK";
        }

        public static string ParseSunsetString(string jsonContent)
        {
            if (!CheckStatus(jsonContent))
            {
                throw new ArgumentException("Status is not valid", nameof(jsonContent));
            }
            dynamic content = JsonConvert.DeserializeObject(jsonContent);
            return content.results.sunset;
        }

        public static DateTime ToLocalTime(DateTime targetDate, string timeString)
        {
            DateTime time = DateTime.Parse(timeString);
            DateTime result = targetDate.Date + time.TimeOfDay;
            return result;
        }
    }

    public interface ISolarService
    {
        string GetServiceData(DateTime date);
    }

    public class SolarService : ISolarService
    {
        public string GetServiceData(DateTime date)
        {
            using (var client = new HttpClient())
            {
                client.BaseAddress = new Uri("http://localhost:8973/api/SolarCalculator");
                client.DefaultRequestHeaders.Accept.Add(
                    new MediaTypeWithQualityHeaderValue("application/json"));
                var apiString = string.Format(
                    "?lat=33.6624&lng=-117.7470&date={0:yyyy-MM-dd}", date);
                HttpResponseMessage response = client.GetAsync(apiString).Result;

                if (response.IsSuccessStatusCode)
                    return response.Content.ReadAsStringAsync().Result;
                return null;
            }
        }
    }
}
